/*
 * TCU (timer/counter unit) support for the X1600.
 *
 * Copyright (C) 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/tcu-common.h>


class Tcu_x1600_channel : public Tcu_channel
{
public:
  explicit Tcu_x1600_channel(l4_addr_t start, uint8_t channel);

  void enable();
  uint8_t get_count_mode();
  void set_count_mode(uint8_t mode);
};


class Tcu_x1600_chip : public Tcu_chip
{
protected:
  unsigned int num_channels()
  { return 8; }

  Tcu_channel *_get_channel(l4_addr_t addr, uint8_t channel);

public:
  explicit Tcu_x1600_chip(l4_addr_t start, l4_addr_t end);
};

#endif /* __cplusplus */



/* C language interface. */

EXTERN_C_BEGIN

void *x1600_tcu_init(l4_addr_t tcu_base, l4_addr_t tcu_base_end);

void *x1600_tcu_get_channel(void *tcu, uint8_t channel);

void x1600_tcu_disable(void *tcu_channel);

void x1600_tcu_enable(void *tcu_channel);

int x1600_tcu_is_enabled(void *tcu_channel);

uint8_t x1600_tcu_get_clock(void *tcu_channel);

void x1600_tcu_set_clock(void *tcu_channel, uint8_t clock);

uint32_t x1600_tcu_get_prescale(void *tcu_channel);

void x1600_tcu_set_prescale(void *tcu_channel, uint32_t prescale);

uint32_t x1600_tcu_get_counter(void *tcu_channel);

void x1600_tcu_set_counter(void *tcu_channel, uint32_t value);

uint8_t x1600_tcu_get_count_mode(void *tcu_channel);

void x1600_tcu_set_count_mode(void *tcu_channel, uint8_t mode);

uint32_t x1600_tcu_get_full_data_value(void *tcu_channel);

void x1600_tcu_set_full_data_value(void *tcu_channel, uint32_t value);

uint32_t x1600_tcu_get_half_data_value(void *tcu_channel);

void x1600_tcu_set_half_data_value(void *tcu_channel, uint32_t value);

EXTERN_C_END
